class Anchors:
    TOP_LEFT = 'tl'
    TOP_CENTER = 'tc'
    TOP_RIGHT = 'tr'
    
    CCENTER = 'cc'
    CENTER_LEFT = 'cl'
    CENTER_RIGHT = 'cr'
    
    BOTTOM_LEFT = 'bl'
    BOTTOM_RIGHT = 'br'
    BOTTOM_CENTER = 'bc'